pro thm_convert_esa_units, data, units, scale=scale

cc3d=findgen(256)

if n_params() eq 0 then return

if strupcase(units) eq strupcase(data.units_name) then return

energy = data.energy           	; in eV                (ne,nbins)
n_e = data.nenergy		; number of energies
nbins=data.nbins		; number of bins
gf = data.geom_factor*data.gf*data.eff
dt = data.integ_t
dt_arr=data.dt_arr		; #dt*#anodes per bin for rate and dead time corrections
mass = data.mass
dead = data.dead		; dead time, (sec) A111
str_element,data,'denom',success=tfdenom

case strupcase(data.units_name) of 
'COMPRESSED' :  scale = 1.d						
'COUNTS' :  scale = 1.d						
'RATE'   :  scale = 1.d*dt*dt_arr
'EFLUX'  :  scale = 1.d*gf 
'FLUX'   :  scale = 1.d*gf * energy
'DF'     :  scale = 1.d*gf * energy^2 * 2./mass/mass*1e5
else: begin
        print,'Unknown starting units: ',data.units_name
	return
      end
endcase

; convert to COUNTS
tmp=data.data
if strupcase(data.units_name) eq 'COMPRESSED' then tmp=cc3d(byte(tmp))
tmp = scale * tmp

; take out dead time correction
if strupcase(data.units_name) ne 'COUNTS' and strupcase(data.units_name) ne 'RATE' and strupcase(data.units_name) ne 'COMPRESSED' then $
	if tfdenom then tmp = round(dt*tmp*data.denom) else tmp = round(dt*tmp/(1.+tmp*dead/dt_arr))
;	tmp = (dt*tmp/(1.+tmp*dead/dt_arr))

scale = 0
case strupcase(units) of
'COMPRESSED' :  scale = 1.
'COUNTS' :  scale = 1.d
'RATE'   :  scale = 1.d/(dt * dt_arr)
'EFLUX'  :  scale = 1.d/(dt * gf)
'FLUX'   :  scale = 1.d/(dt * gf * energy)
'DF'     :  scale = 1.d/(dt * gf * energy^2 * 2./mass/mass*1e5 )
else: begin
        message,'Undefined units: '+units
        return
      end
endcase

; dead time correct data if not counts or rate
if strupcase(units) ne 'COUNTS' and strupcase(units) ne 'RATE' and strupcase(units) ne 'COMPRESSED' then begin

; the following is to correct for high count rates in solar wind ions 
;	where standard dead time correction doesn't work due to averaging
;	corr1 and corr2 account for changes in rate during an energy accumulation
;	cdt_arr accounts for beams that fall on a subset of the anodes 
;	default is	denom = 1.- dead/dt_arr*tmp/dt


	if data.charge eq 1. then begin
		if ndimen(tmp) eq 1 then begin

			cdt_arr=dt_arr
			spec = tmp
			if max(spec,ms) gt total(spec)/4. then begin
				rs = (spec(ms)-(spec(ms-1)+spec(ms+1))/2.)/spec(ms)
				re = (energy(ms-1)-energy(ms+1))/energy(ms)
				rth = re*!radeg/5.625
				cdt_arr(ms-1:ms+1) = (8.-1.0*rs*rth) > 4.7
;print,'cdt_arr min max',minmax(cdt_arr)
			endif 

			tmp2 = data.bins*tmp

			slp = tmp2[0:n_e-2]-tmp2[1:n_e-1]
			avg = ((tmp2[0:n_e-2]+tmp2[1:n_e-1])>1.)

			corr1 = fltarr(n_e) & corr1[*]=1. & corr2=corr1
			corr_exp = 2.

			corr1[1:n_e-2] = 1. - .15*((abs(slp[0:n_e-3])+abs(slp[1:n_e-2]))/(avg[0:n_e-3]+avg[1:n_e-2]))^0.5
			corr2[1:n_e-2] = 1. - .50*(abs(slp[0:n_e-3]+slp[1:n_e-2])/(avg[0:n_e-3]+avg[1:n_e-2]))

			denom = 1.- dead/cdt_arr*tmp/(dt*corr1*corr2)

		endif else begin

			cdt_arr=dt_arr
			spec = total(tmp,2)
			if min(data.dtheta) ne 5.625 and max(spec,ms) gt total(spec)/4. then begin
				rs = (spec(ms)-(spec(ms-1)+spec(ms+1))/2.)/spec(ms)
				re = (energy(ms-1,0)-energy(ms+1,0))/energy(ms,0)
				rth = re*!radeg/5.625
				sld = total(tmp,1)
				ind2 = where(sld gt total(sld)/10.)
				cdt_arr(ms-1:ms+1,ind2) = (8.-1.0*rs*rth) > 4.7
;				cdt_arr(ms-1:ms+1,ind2) = (8.-1.8*rs*rth) > 2.
;print,'cdt_arr min max',minmax(cdt_arr)
			endif 

			tmp2 = data.bins*tmp

			slp = tmp2[0:n_e-2,*]-tmp2[1:n_e-1,*]
			avg = ((tmp2[0:n_e-2,*]+tmp2[1:n_e-1,*])>1.)

			
			corr1 = fltarr(n_e,nbins) & corr1[*,*]=1. & corr2=corr1
			corr_exp = 2.

			corr1[1:n_e-2,*] = 1. - .15*((abs(slp[0:n_e-3,*])+abs(slp[1:n_e-2,*]))/(avg[0:n_e-3,*]+avg[1:n_e-2,*]))^0.5
			corr2[1:n_e-2,*] = 1. - .50* ( abs(slp[0:n_e-3,*] +    slp[1:n_e-2,*]) /(avg[0:n_e-3,*]+avg[1:n_e-2,*]))

			denom = 1.- dead/cdt_arr*tmp/(dt*corr1*corr2)

		endelse
	endif else denom = 1.- dead/dt_arr*tmp/dt

; check on solar wind correction
	denom2 = 1.- dead/dt_arr*tmp/dt
	ind2 = where(abs(denom2-denom) gt .01, count)
	if count ne 0 then begin
;		print,'***********Solar wind dead time correction modification.**************'
;		print,count
;		for i=0,count-1 do print,i,ind2[i],denom2[ind2[i]]-denom[ind2[i]],denom2[ind2[i]],denom[ind2[i]],cdt_arr[ind2[i]],corr1[ind2[i]],corr2[ind2[i]],tmp[ind2[i]]
;		print,'***********Solar wind dead time correction modification.**************'
	endif


; set a maximum limit on the dead time correction
	void = where(denom lt .1,count)
	if count gt 0 then begin
		dprint,dlevel=1,min(denom,ind)
		denom = denom>.01 
;		denom = denom>.1 
		dprint,dlevel=1,' Error: convert_peace_units dead time error.'
		dprint,dlevel=1,' Dead time correction limited to x10 for ',count,' bins'
		dprint,dlevel=1,' Time= ',time_string(data.time,/msec)
	endif
	tmp2 = tmp/denom

	str_element,data,'denom',denom

endif else tmp2 = tmp

; scale to new units

data.units_name = units
if find_str_element(data,'ddata') ge 0 then data.ddata = scale * tmp2^.5

if strupcase(units) eq 'COMPRESSED' then begin
	ind=where(tmp2 ge 0.,npts)
	ntmp=n_elements(tmp2)
	if npts gt 0 then begin
		for i=0,npts do begin
			minval=min(abs(cc3d-tmp2(ind(i))),jj)
			tmp2(ind(i))=jj
		endfor
	endif
	if npts ne ntmp then begin
		tmp3=intarr(ntmp) 
		if npts gt 0 then tmp3(ind)=1 
		ind2=where(tmp3 eq 0)
		tmp2(ind2)=255
	endif
endif

data.data = scale * tmp2

return
end



