
;+
;NAME:
; thm_sst_dist3d_16x64
;PURPOSE:
;  This routine defines the appropriate distribution representation struct for 
;  16 Energy and 64 angle SST data.
;  
;
;$LastChangedBy: pcruce $
;$LastChangedDate: 2010-12-16 17:26:18 -0800 (Thu, 16 Dec 2010) $
;$LastChangedRevision: 7929 $
;$URL: svn+ssh://thmsvn@ambrosia2.ssl.berkeley.edu/repos/thmsoc/tags/tdas_6_00_vmtest/idl/themis/spacecraft/particles/SST/SST_cal_workdir/thm_sst_dist3d_16x64_2__define.pro $
;-


pro thm_sst_dist3d_16x64_2__define,structdef=dat

nenergy = 16
nbins = 64
dims = [nenergy,nbins]

dat = {thm_sst_dist3d_16x64_2, inherits dist3d, $
       mode:                 0    , $
       cnfg:    0   ,$
       nspins:               0    ,  $
       data:    fltarr(dims)      , $  ;the actual data is stored here
       energy:  fltarr(dims)    , $    ;the energy value midpoints at which each data bin is measured are stored here
       theta:   fltarr(dims)    , $    ;the theta angle midpoints at which each data bin is measured are stored here(theta is elevation out of the spacecraft spin plane)
       phi:     fltarr(dims)    , $    ;the phi angle midpoints at which each data bin is measured are stored here (phi is rotation within the spacecraft spin plane)
       denergy: fltarr(dims)      , $  ;the change in energy across each bin.  (ie bin start/stop are energy +- denergy/2)
       dtheta:  fltarr(dims)    , $    ;the change in theta across each bin.  (ie bin start/stop are theta +- dtheta/2)
       dphi:    fltarr(dims)    , $    ;the change in phi across each bin.  (ie bin start/stop are phi +- dphi/2)
 ;      domega:  fltarr(dims)     ,$
       bins:    intarr(dims)     ,$
       gf:      fltarr(dims)     ,$  ;the geometric factor correction for each bin
       integ_t: fltarr(dims)    , $  ;the intergration time for each bin(in seconds)
       deadtime:fltarr(dims)    , $  ;the deadtime correction factor for each bin
       geom_factor: !values.f_nan  ,  $ ;the nominal(theoretical) geometric factor for all bins
       atten:   -1,   $                 ;whether the attenuator is on or off 
       att:fltarr(dims), $              ;attenuator scaling factors, applied when attenuator is on 
       eff: fltarr(dims)+1,$            ;detector efficiencies.
       energy_scales:fltarr(4)+1,$      ;DAP bin number scaling factors to convert into eV
       energy_offsets:fltarr(4), $      ;scaled DAP bin number offsets factors to convert into eV
       energy_reciprocals:fltarr(4), $  ;scaling factors for reciprocal of DAP bin number to convert into eV
       dead: 0 $ ;the nominal deadtime correction for all bins
       }
dat.nenergy = nenergy
dat.nbins = nbins

end

