
;+
;NAME:
; thm_sst_energy_cal2
;PURPOSE:
;  This routine performs bin energy boundary calibrations.  Transforming DAP values into eV
;  The routine directly modifies the dat struct according to parameters stored in the struct
;  But results can be returned for reference.
;  
;Inputs:
;  dat
;  
;Outputs:
;  energy:  The midpoint of each energy bin
;  denergy: The width of each energy bin
;
;SEE ALSO:
;  thm_sst_convert_units2
;
;$LastChangedBy: pcruce $
;$LastChangedDate: 2010-12-16 17:26:18 -0800 (Thu, 16 Dec 2010) $
;$LastChangedRevision: 7929 $
;$URL: svn+ssh://thmsvn@ambrosia2.ssl.berkeley.edu/repos/thmsoc/tags/tdas_6_00_vmtest/idl/themis/spacecraft/particles/SST/SST_cal_workdir/thm_sst_energy_cal2.pro $
;-


pro thm_sst_energy_cal2,dat,energy=energy,denergy=denergy

;if inst eq 1 then begin
  dap_borders = [12,19,26,34,44,69,103,150,215,306,506,906,2000,3000,4000,5000,60000]  ; last 4 are fill!

  one = replicate(1.,16)
  e_start =  (((1./dap_borders[0:15]) # dat.energy_reciprocals) + (dap_borders[0:15] # dat.energy_scales)  + (one # dat.energy_offsets)) * 1000.
  e_end   =  (((1./dap_borders[0:15]) # dat.energy_reciprocals) + (dap_borders[1:16] # dat.energy_scales)  + (one # dat.energy_offsets)) * 1000.
  denergy    = e_end - e_start
  energy   = (e_end + e_start)/2
  detectors = [[replicate(0,16)],[replicate(1,16)],[replicate(2,16)],[replicate(3,16)]]
 
  dat.energy = energy[*,detectors]
  dat.denergy = denergy[*,detectors]
  
end
