;+
;NAME:
; thm_sst_read_calibration_params
;PURPOSE:
;  This routine reads calibration parameters from a file.
;
;Inputs:
;  thx:  probe identifier string (e.g. 'tha','thb'...)
;  species: 'e' or 'i', to indicate ion or electron
;  dtype: distribution type 'f','b','r'
;
;Returns:
;  The parameter struct for the calibration file
;    
;NOTES:
;  #1 Reading and applying the calibration parameters is done separately, because reading later when 
;     they are applied leads to duplicated parameter reads. And because there is not enough information
;     to apply parameters at load time.
;
;$LastChangedBy: pcruce $
;$LastChangedDate: 2010-07-15 14:05:27 -0700 (Thu, 15 Jul 2010) $
;$LastChangedRevision: 7712 $
;$URL: svn+ssh://thmsvn@ambrosia2.ssl.berkeley.edu/repos/thmsoc/tags/tdas_6_00_vmtest/idl/themis/spacecraft/particles/SST/SST_cal_workdir/thm_sst_read_calib_params.pro $
;-


function thm_sst_read_calib_params,thx,species,dtype

  ;full distribution parameters used for all data types atm
  file = file_retrieve(strlowcase(thx)+'/l1/sst/0000/'+strlowcase(thx)+'_ps'+strlowcase(species)+'f_calib_params.txt',_extra=!themis)
  dprint,'Reading cal file:', file

  param_struct = read_asc(file)
  
  if ~is_struct(param_struct) then begin
    dprint,'Error reading calibration file: ' + file,dlevel=0
    return,0
  endif
  
  return,param_struct
end