;+
;thm_ui_draw_object method: addLegend
;
;This routine adds a legend to a particular view/panel.
;
;Inputs:
;  view(IDLgrView):  The view to which the legend static components are added
;  annotation(IDLgrView): The view to which the legend dynamic components are added
;  panelInfo(struct):  This is the struct that stores all the information about the panel
;  traceInfoArray(array if structs):  This is an array of structs that store all the
;                                     info for all the traces in this panel
; 
;NOTE this routine is currently automatically generating labels
;IT SHOULDN'T BE DOING THIS.  When default labels are being
;correctly set, it should just use the label text object from
;the appropriate axis.
;
;$LastChangedBy: pcruce $
;$LastChangedDate: 2010-08-25 18:05:01 -0700 (Wed, 25 Aug 2010) $
;$LastChangedRevision: 7774 $
;$URL: svn+ssh://thmsvn@ambrosia2.ssl.berkeley.edu/repos/thmsoc/tags/tdas_6_00_vmtest/idl/themis/thm_ui_new/display/draw_object/thm_ui_draw_object__addlegend.pro $
;-
pro thm_ui_draw_object::addLegend,view,annotation,panelInfo,traceInfoArray

  compile_opt idl2
  
  textsize = 11  ;Base-text size for auto generated labels
  spacing = 2. ;Default vertical spacing between lines in legend
  zstack = .6  ;60% height of legend by default. Puts legend on top of most things
  
  numBias = 10 ; number of chars that annotation will take if using numerical output
  timeBias = 17 ; number of chars that annotation will take if using time output
  extraBias = 3 ; number of chars colon and space will take
  
  nchars = 0  ;Intialize the counter for the number of chars in the longest line
  nLines = 2+n_elements(traceInfoArray) ; The number of lines in the legend
 
  ;calculate the size of legend in chars
  ;the the legend should be as large as the most
  ;characters in any line
  ;Here the bias is the number of characters that will be
  ;contributed by the annotated value itself.(as opposed to the label)
  if panelInfo.xIsTime then begin
    text = 'X Axis Time'
    bias = timeBias
  endif else begin
    text = 'X Axis Value'
    bias = numBias
  endelse
  
  bias += extraBias
  
  nChars = nChars > (strlen(text)+bias)
  
  if panelInfo.yIsTime then begin
    text = 'Y Axis Time'
    bias = timeBias
  endif else begin
    text = 'Y Axis Value'
    bias = numBias
  endelse
  bias += extraBias
  nChars = nChars > (strlen(text)+bias)
    
  if panelInfo.hasSpec then begin
    if panelInfo.zisTime then begin
      bias = timeBias
    endif else begin
      bias = numBias
    endelse
    bias+= extraBias
  endif else begin
    if panelInfo.yistime then begin
      bias = timeBias
    endif else begin
      bias = numBias
    endelse
    bias+= extraBias
  endelse
  
  ;Identify the maximum number of characters in any line
  for i = 0,n_elements(traceInfoArray)-1 do begin
    nChars = nChars > (strlen(traceInfoArray[i].dataName)+bias)
  endfor
  
  ;2 is for colon and one extra
  ; nChars+=annoBias+extrabias
  
  xdiv = panelInfo.xplotpos[1]-panelInfo.xplotpos[0]
  ydiv = panelInfo.yplotpos[1]-panelInfo.yplotpos[0]
  
  ;these estimates assume that a character width is 1/2 its height
  ;this may not be accurate across systems
  width = self->pt2norm(textsize * nchars/2 + 2*spacing,0)/xdiv
  height = self->pt2norm(nlines * textsize + (nlines+1)*spacing,1)/ydiv
  
  ;if the legend is going to be outside of the view, we need to calculate a bias
  ;to shift it down/left so that it will not be clipped.
  view->getProperty,viewPlane_rect=vpr
  if width/2D gt vpr[2]+vpr[0]-1D then begin
    wbias = width/2D - (vpr[2]+vpr[0]-1D)
  endif else begin
    wbias = 0D
  endelse
  
  ;ensure that legend is not flush with edge of screen
  wbias += self->pt2norm(2.,0)/xdiv
  
  if height/2D gt vpr[3]+vpr[1]-1D then begin
    hbias = height/2D - (vpr[3]+vpr[1]-1D)
  endif else begin
    hbias = 0D
  endelse

  ;If the legend is turned on, then don't make it hidden
  if self.legendOn eq 1 || self.legendOn eq 2 then begin
    hide_val = 0
  endif else begin
    ;Even if turned off, still draw it and hide it, so we
    ;can turn it back on without a call to update
    hide_val = 1
  endelse
  
  model = obj_new('IDLgrModel',hide=hide_val)
  anno_model = obj_new('IDLgrModel',hide=hide_val)
  
  ;Legend background
  polygon = obj_new('IDLgrPolygon',$
    [1.-width/2.,1+width/2.,1+width/2.,1-width/2.]-wbias,$
    [1.-height/2.,1-height/2.,1+height/2.,1+height/2.]-hbias,$
    [zstack,zstack,zstack,zstack],color=self->convertColor([255,255,255]),/double)
    
  ;Legend border
  polyline = obj_new('IDLgrPolyline',$
    [1.-width/2.,1+width/2.,1+width/2.,1-width/2.,1.-width/2.]-wbias,$
    [1.-height/2.,1-height/2.,1+height/2.,1+height/2.,1-height/2.]-hbias,$
    [zstack,zstack,zstack,zstack,zstack]+.00001,color=self->convertColor([0,0,0]),/double)
    
  model->add,polyline
  model->add,polygon
  
  ;Spacing in normalized coordinates
  xspaceNorm = self->pt2norm(spacing,0)/xdiv
  yspaceNorm = self->pt2norm(spacing,1)/ydiv
  xcharNorm = self->pt2norm(textsize,0)/xdiv
  ycharNorm = self->pt2norm(textsize,1)/ydiv
  
  labelLocation=[1.-width/2. + xspacenorm - wbias,1.+height/2 - yspacenorm - ycharnorm - hbias,zstack+.05]
  valueLocation=[1.+width/2. - xspacenorm - wbias,1.+height/2 - yspacenorm - ycharnorm - hbias,zstack+.05]
  
  ;Different label depending on whether x is a time or not
  if panelInfo.xIsTime then begin
    text = 'X Axis Time'
  endif else begin
    text = 'X Axis Value'
  endelse
  
  ;space pad to line up colons
  text += ' :'
  
  ;Generate label text object
  thm_label = obj_new('thm_ui_text',value=text,size=textsize,font=2)
  xlabel = self->getTextObject(thm_label,labelLocation,1,0,justify=-1)
  model->add,xlabel
  
  ;Generate text object where values will be drawn
  thm_value = obj_new('thm_ui_text',value='',size=textsize,font=2)
  xvalue = self->getTextObject(thm_value,valueLocation,1,0,justify=1,/enable_formatting)
  panelInfo.xobj = xvalue
  anno_model->add,xvalue

  labelLocation[1] -= (yspacenorm+ycharnorm)
  valueLocation[1] -= (yspacenorm+ycharnorm)
  
  ;Different label depending on whether y is a time ro not
  if panelInfo.yIsTime then begin
    text = 'Y Axis Time'
  endif else begin
    text = 'Y Axis Value'
  endelse
  
  text += ' :'
  
  ;Generate labe
  thm_label = obj_new('thm_ui_text',value=text,size=textsize,font=2)
  ylabel = self->getTextObject(thm_label,labelLocation,1,0,justify=-1)
  model->add,ylabel
  
  ;Generate text object where values will be drawn
  thm_value = obj_new('thm_ui_text',value='',size=textsize,font=2)
  yvalue = self->getTextObject(thm_value,valueLocation,1,0,justify=1,/enable_formatting)
  panelInfo.yobj = yvalue
  anno_model->add,yvalue

  ; ycnt = 1
  ; zcnt = 1
  
  ;loop backwards because traces are in reverse order to create proper layering
  for i = n_elements(traceInfoArray)-1,0,-1 do begin
   ; if traceInfoArray[i].isSpec then begin
  
      labelLocation[1] -= (yspacenorm+ycharnorm)
      valueLocation[1] -= (yspacenorm+ycharnorm)
      
      text = traceInfoArray[i].dataName + ' :'
      
      ;Label for each trace
      thm_label = obj_new('thm_ui_text',value=text,size=textsize,font=2,color=traceInfoArray[i].color)
      olabel = self->getTextObject(thm_label,labelLocation,1,0,justify=-1)
      model->add,olabel
      
      ;Value text object for each trace
      thm_value = obj_new('thm_ui_text',value='',size=textsize,font=2,color=traceInfoArray[i].color)
      ovalue = self->getTextObject(thm_value,valueLocation,1,0,justify=1,/enable_formatting)
      traceInfoArray[i].textobj = ovalue
      anno_model->add,ovalue
  ;    thm_text = obj_new('thm_ui_text',value=text,size=textsize,font=2)
   ; endif
    
  endfor
  
  ;add to info 
  panelInfo.legendModel = model
  panelInfo.legendAnnoModel = anno_model
  
  ;If everything complete successfully add models to views
  view->add,model
  annotation->add,anno_model
  
end