;+ 
;NAME:  
;  thm_ui_tplot_gui_load_tvars
;  
;PURPOSE:
;  Verifies input tplot variables for tplot_gui.  This is actually the trickiest part of the whole process
;  
;CALLING SEQUENCE:
; thm_ui_tplot_gui_validate_tvars,in_names,out_names=out_names
;  
;INPUT:
;  in_names:  Input names supplied by user.
;  
;Keywords:
;(Input):
;  no_verify: value of the no_verify keyword supplied by user(optionally)
;(Output):
;  out_names: csvector containing lists of validated names for plots
;  all_names: All the non-pseudo variables that will be used for verification
;  limits: csvector containing variable specific settings, somewhat collated in the case of pseudo variables, 
;  dlimits: csvector containing defaultish variable specific settings, somewhat collated in the case of pseudo variables
;  
;
;SEE ALSO:
;  ssl_general/misc/csvector.pro
;  (Used to generate linked list, so that temporary tplot variables aren't needed)
;  
;$LastChangedBy: pcruce $
;$LastChangedDate: 2010-04-30 16:49:56 -0700 (Fri, 30 Apr 2010) $
;$LastChangedRevision: 7571 $
;$URL: svn+ssh://thmsvn@ambrosia2.ssl.berkeley.edu/repos/thmsoc/tags/tdas_6_00_vmtest/idl/themis/thm_ui_new/utilities/thm_ui_tplot_gui_funcs/thm_ui_tplot_gui_load_tvars.pro $
;--------------------------------------------------------------------------------

pro thm_ui_tplot_gui_load_tvars,in_names,no_verify=no_verify,out_names=out_names,all_names=all_names

  compile_opt idl2
    
  ;just in case
  undefine,out_names
  undefine,all_names
  undefine,limits
  undefine,dlimits
  
  ; make sure indexes are converted array of tplot variable names
  varnames = tnames(in_names,nd,/all)
  
  if nd eq 0 then begin
     return
  endif
    
  if keyword_set(no_verify) then begin
    clobber = 'yesall'
  endif else begin
    clobber = ''
  endelse
    
  ; add tplot variables to loadedData object
  for i=0L,nd-1 do begin
  
    ; get pre-existing gui variable names
    guiNames = !thm_gui_new.loadedData->GetAll(/Parent)
    if size(guiNames, /type) ne 7 then guiNames=''
  
    ; check if pseudovariable
    get_data, varnames[i], data=d, dlimits=pseudo_dl, limits=pseudo_l
    dSize = size(d, /type)
    
    if dSize eq 7 then begin
    ;load tplot pseudovariable
    
      ;extract_tags, pseudo_dl, pseudo_l
      
      subNames = tnames(d, sub_nd, ind=sub_ind, /all)
      undefine,valid_sub
 
      for j=0L,sub_nd-1 do begin
      ; load each component of pseudovariable
      
      
        ;make sure dlimits from pseudovar are inherited by component variables
        get_data, subNames[j], dlimits=sub_dl, limits=sub_l,data=d
        extract_tags, sub_dl, pseudo_dl
        extract_tags, sub_l, pseudo_l

        if (clobber ne 'yesall' AND clobber ne 'noall') AND in_set(subNames[j], guiNames) then begin
          clobber=thm_ui_load_clob_prompt(!thm_gui_new.guiId, !thm_gui_new.historyWin, $
                                          subNames[j])
          ; loop so that hitting the widget kill button doesn't close the window
          while clobber eq '' do clobber=thm_ui_load_clob_prompt(!thm_gui_new.guiId, $
                                                                 !thm_gui_new.historyWin, $
                                                                 subNames[j])
        endif 
        
        if clobber eq 'yes' OR clobber eq 'yesall' then begin
          h = 'TPLOT_GUI: ' + strupcase(subNames[j]) + ' will be overwritten.'
          !thm_gui_new.historyWin->Update, h
        endif
        
        if (clobber eq 'no' OR clobber eq 'noall') AND in_set(subNames[j], guiNames) then begin
          h = 'LOAD DATA: ' + strupcase(subNames[j]) + $
              ' not loaded to prevent overwrite of existing data.'
          !thm_gui_new.historyWin->Update, h
        endif else if ~!thm_gui_new.loadedData->addData(subNames[j],d,limit=sub_l,dlimit=sub_dl) then begin
          dprint,"Problem adding: " + subNames[j] + " to GUI"        
        endif
        
        valid_sub = array_concat(subNames[j],valid_sub)   
        
      endfor
      
      if n_elements(valid_sub) gt 0 then begin
        all_names = array_concat(valid_sub,all_names)
        out_names = csvector(valid_sub,out_names)        
      endif
  
    endif else begin
    ; load standard tplot variable
      if (clobber ne 'yesall' AND clobber ne 'noall') AND in_set(varnames[i], guiNames) then begin
        clobber=thm_ui_load_clob_prompt(!thm_gui_new.guiId, !thm_gui_new.historyWin, $
                                        varnames[i])
        ; loop so that hitting the widget kill button doesn't close the window
        while clobber eq '' do clobber=thm_ui_load_clob_prompt(!thm_gui_new.guiId, $
                                                               !thm_gui_new.historyWin, $
                                                               varnames[i])
      endif
      
      if clobber eq 'yes' OR clobber eq 'yesall' then begin
        h = 'TPLOT_GUI: ' + strupcase(varnames[i]) + ' will be overwritten.'
        !thm_gui_new.historyWin->Update, h
      endif
      
      if (clobber eq 'no' OR clobber eq 'noall') AND in_set(varnames[i], guiNames) then begin
        h = 'LOAD DATA: ' + strupcase(varnames[i]) + $
            ' not loaded to prevent overwrite of existing data.'
        !thm_gui_new.historyWin->Update, h
      endif else if ~!thm_gui_new.loadedData->add(varnames[i]) then begin
        dprint,"Problem adding: " + varnames[i] + " to GUI"
      endif
      
        all_names = array_concat(varnames[i],all_names)
        out_names = csvector(varnames[i],out_names)
    
    endelse
  endfor
  
end